(*
////////////////////////////////////////////////////////////////////////////////
// FIELDGRD: A DBGrid with Defineable Fields                                  //
// This code is Copyright  1996 Steve Flynn, All Rights Reserved.            //
// This code may be distributed as Freeware providing all original files are  //
// included and the Copyright message above is not altered.                   //
////////////////////////////////////////////////////////////////////////////////
*)

unit FieldGrd;

interface

uses
	Classes, Controls, StdCtrls, Grids, DBGrids, Graphics, WinTypes,
WinProcs;

type
	TDBFieldGrid = class(TDBGrid)
	private
		FFieldList: TStrings;
		FWidthList: TStrings;
		FTitleList: TStrings;

	protected
		procedure SetFieldList(Value: TStrings);
		procedure SetWidthList(Value: TStrings);
		procedure SetTitleList(Value: TStrings);
		procedure SetColumnAttributes; override;
		procedure Paint; override;
		procedure DefineFieldMap; override;

	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;

	published
		property ColumnFields: TStrings read FFieldList write SetFieldList;
		property ColumnWidths: TStrings read FWidthList write SetWidthList;
		property ColumnTitles: TStrings read FTitleList write SetTitleList;
		property ScrollBars;
		end;

procedure Register;


implementation

uses
	SysUtils, DBConsts, LibConst;

procedure Register;
	begin
	RegisterComponents(LoadStr(srDControls), [TDBFieldGrid]);
	end;


function Min(a, b: integer): integer;
	begin
	if b < a then
		Result := b
	else
		Result := a;
	end;


procedure RaiseGridError(const S: string);
	begin
	raise EInvalidGridOperation.Create(S);
	end;


constructor TDBFieldGrid.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
	FFieldList := TStringList.Create;
	FWidthList := TStringList.Create;
	FTitleList := TStringList.Create;
	end;


destructor TDBFieldGrid.Destroy;
	begin
	FTitleList.Free;
	FWidthList.Free;
	FFieldList.Free;
	inherited Destroy;
	end;


procedure TDBFieldGrid.SetFieldList(Value: TStrings);
	begin
	FFieldList.Assign(Value);
	LayoutChanged;
	end;


procedure TDBFieldGrid.SetWidthList(Value: TStrings);
	begin
	FWidthList.Assign(Value);
	LayoutChanged;
	end;


procedure TDBFieldGrid.SetTitleList(Value: TStrings);
	begin
	FTitleList.Assign(Value);
	LayoutChanged;
	end;


procedure TDBFieldGrid.DefineFieldMap;
	var
	i: integer;
	begin
	if not Visible then
		exit;
	for i := 0 to FFieldList.Count - 1 do
		if not (Datalink.AddMapping(FFieldList[i])) then
			RaiseGridError(FmtLoadStr(SFieldNotFound, [FFieldList[i]]));
	end;


procedure TDBFieldGrid.Paint;
	begin
	if not (ScrollBars in [ssVertical, ssBoth]) then
		SetScrollRange(Handle, SB_VERT, 0, 0, False);
	if not (ScrollBars in [ssHorizontal, ssBoth]) then
		SetScrollRange(Handle, SB_HORZ, 0, 0, False);
	inherited Paint;
	end;


procedure TDBFieldGrid.SetColumnAttributes;
	var
	i, CWidth: integer;
	begin
	for i := 0 to Min(FieldCount - 1, Min(FTitleList.Count - 1, FWidthList.Count - 1)) do
		begin
		Fields[i].DisplayLabel := FTitleList[i];
		Fields[i].DisplayWidth := StrToInt(FWidthList[i]);
		Canvas.Font := Font;
		CWidth := Fields[i].DisplayWidth * Canvas.TextWidth('0') + 4;
		ColWidths[I + Ord(dgIndicator in Options)] := CWidth;
		TabStops[I + Ord(dgIndicator in Options)] := not ReadOnly and not Fields[i].Calculated;
		end;
	end;

end.

